/****************************************************************************
**
** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the examples of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** No Commercial Usage
** This file contains pre-release code and may not be distributed.
** You may use this file in accordance with the terms and conditions
** contained in the Technology Preview License Agreement accompanying
** this package.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights.  These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** If you have questions regarding the use of this file, please contact
** Nokia at qt-info@nokia.com.
**
**
**
**
**
**
**
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QtGui>
#include <QtDebug>

#include "window.h"

Window::Window(QWidget *parent, bool stacked)
    : QWidget(parent, Qt::Window), m_level(-1), m_dialog(0)
{
    if (stacked) {
        m_level = 0;
        setAttribute(Qt::WA_Maemo5StackedWindow);
        if (Window *w = ::qobject_cast<Window *>(parent)) {
            m_level = w->m_level+1;
        }
    }
    QMenuBar *bar = new QMenuBar(this);
    bar->addAction(tr("Open new window"), this, SLOT(openNewWindow()));
    bar->addAction(tr("Show a MessageBox"), this, SLOT(showMessageBox()));
    bar->addAction(tr("Close this window"), this, SLOT(close()));

    QVBoxLayout *lay = new QVBoxLayout(this);
    lay->addWidget(new QLabel(tr("<qt>This is a Maemo5 Stacked Window. Current stacking depth is %1</qt>").arg(m_level)));
    
    QPushButton *pb = new QPushButton(tr("Open test dialog"), this);
    lay->addWidget(pb);
    connect(pb, SIGNAL(clicked()), this, SLOT(showDialog()));

    setWindowTitle(tr("Stacked Windows [%1]").arg(m_level));
}

Window::~Window()
{
    qWarning() << "Stacked Window [" << m_level << "] is being destroyed.";
}

void Window::openNewWindow()
{
    Window *w = new Window(this, true);
    w->show();
}

void Window::showDialog()
{
    if (!m_dialog) {
        m_dialog = new QDialog(this);
        m_dialog->setWindowTitle(tr("A Dialog"));
        
        QVBoxLayout *lay = new QVBoxLayout(m_dialog);
        lay->addWidget(new QLabel(tr("<qt>This is a Maemo5 Stacked Dialog.")));
    }
    if (!m_dialog->isVisible())
        m_dialog->show();
}

void Window::showMessageBox()
{
    QMessageBox::information(this, tr("A MessageBox"), tr("This is a Maemo5 Message Box."), QMessageBox::Ok | QMessageBox::Cancel | QMessageBox::Apply);
}
