/****************************************************************************
**
** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
** Contact: Qt Software Information (qt-info@nokia.com)
**
** This file is part of the QtGui module of the unofficial Maemo Qt Toolkit.
**
** No Commercial Usage
** This file contains pre-release code and may not be distributed.
** You may use this file in accordance with the terms and conditions
** contained in the either Technology Preview License Agreement or the
** Beta Release License Agreement.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License versions 2.0 or 3.0 as published by the Free
** Software Foundation and appearing in the file LICENSE.GPL included in
** the packaging of this file.  Please review the following information
** to ensure GNU General Public Licensing requirements will be met:
** http://www.fsf.org/licensing/licenses/info/GPLv2.html and
** http://www.gnu.org/copyleft/gpl.html.  In addition, as a special
** exception, Nokia gives you certain additional rights. These rights
** are described in the Nokia Qt GPL Exception version 1.3, included in
** the file GPL_EXCEPTION.txt in this package.
**
** Qt for Windows(R) Licensees
** As a special exception, Nokia, as the sole copyright holder for Qt
** Designer, grants users of the Qt/Eclipse Integration plug-in the
** right for the Qt/Eclipse Integration to link to functionality
** provided by Qt Designer and its related libraries.
**
** If you are unsure which license is appropriate for your use, please
** contact the sales department at qt-sales@nokia.com.
**
****************************************************************************/

#include <qgtkstyle.h>
#include <qmaemo5style.h>
#include <private/qmaemo5style_p.h>

#include <private/qapplication_p.h>

#include <QtCore/QLibrary>
#include <QtGui/QStyleOption>
#include <QtGui/QLineEdit>
#include <QtGui/QX11Info>

#include "qpixmapcache.h"
#undef signals // Collides with GTK stymbols
#include "qgtkpainter_p.h"

#include <X11/Xlib.h>

#if defined(Q_WS_MAEMO_5)

QT_BEGIN_NAMESPACE

Ptr_gtk_widget_set_name QMaemo5StylePrivate::gtk_widget_set_name = 0;
Ptr_gtk_text_view_new QMaemo5StylePrivate::gtk_text_view_new = 0;
Ptr_gtk_widget_get_size_request QMaemo5StylePrivate::gtk_widget_get_size_request = 0;

Ptr_hildon_init QMaemo5StylePrivate::hildon_init = 0;
Ptr_hildon_entry_new QMaemo5StylePrivate::hildon_entry_new = 0;
Ptr_hildon_number_editor_new QMaemo5StylePrivate::hildon_number_editor_new = 0;
Ptr_hildon_app_menu_new QMaemo5StylePrivate::hildon_app_menu_new = 0;
Ptr_hildon_button_new QMaemo5StylePrivate::hildon_button_new = 0;
Ptr_hildon_gtk_widget_set_theme_size QMaemo5StylePrivate::hildon_gtk_widget_set_theme_size = 0;

Ptr_hildon_file_chooser_dialog_new QMaemo5StylePrivate::hildon_file_chooser_dialog_new = 0;
Ptr_hildon_file_chooser_dialog_set_extension QMaemo5StylePrivate::hildon_file_chooser_dialog_set_extension = 0;
Ptr_hildon_file_chooser_dialog_add_extensions_combo QMaemo5StylePrivate::hildon_file_chooser_dialog_add_extensions_combo = 0;

void QMaemo5StylePrivate::resolveGtk()
{
    QGtkStylePrivate::resolveGtk();

    QLibrary libgtk(QLS("gtk-x11-2.0"), 0, 0);
    QLibrary libhildon(QLS("hildon-1"), 0, 0);
    QLibrary libhildonfm(QLS("hildonfm"), 2, 0);

    gtk_widget_set_name = (Ptr_gtk_widget_set_name)libgtk.resolve("gtk_widget_set_name");
    gtk_text_view_new  = (Ptr_gtk_text_view_new)libgtk.resolve("gtk_text_view_new");
    gtk_widget_get_size_request  = (Ptr_gtk_widget_get_size_request)libgtk.resolve("gtk_widget_get_size_request");

    hildon_init = (Ptr_hildon_init)libhildon.resolve("hildon_init");
    hildon_entry_new = (Ptr_hildon_entry_new)libhildon.resolve("hildon_entry_new");
    hildon_number_editor_new = (Ptr_hildon_number_editor_new)libhildon.resolve("hildon_number_editor_new");
    hildon_app_menu_new = (Ptr_hildon_app_menu_new)libhildon.resolve("hildon_app_menu_new");
    hildon_button_new = (Ptr_hildon_button_new)libhildon.resolve("hildon_button_new");
    hildon_gtk_widget_set_theme_size = (Ptr_hildon_gtk_widget_set_theme_size)libhildon.resolve("hildon_gtk_widget_set_theme_size");

    hildon_file_chooser_dialog_new = (Ptr_hildon_file_chooser_dialog_new)libhildonfm.resolve("hildon_file_chooser_dialog_new");
    hildon_file_chooser_dialog_set_extension = (Ptr_hildon_file_chooser_dialog_set_extension)libhildonfm.resolve("hildon_file_chooser_dialog_set_extension");
    hildon_file_chooser_dialog_add_extensions_combo = (Ptr_hildon_file_chooser_dialog_add_extensions_combo)libhildonfm.resolve("hildon_file_chooser_dialog_add_extensions_combo");

}

QString QMaemo5StylePrivate::getThemeName() const
{
    QString rcPaths = QString::fromLocal8Bit(qgetenv("GTK2_RC_FILES"));
    if (!rcPaths.isEmpty()) {
        QStringList paths = rcPaths.split(QLatin1String(":"));
        foreach (const QString &rcPath, paths) {
            if (rcPath.startsWith(QLatin1String("/etc/hildon/theme/gtk-2.0/gtkrc"))) {
                QString slt = QFile::symLinkTarget(QLatin1String("/etc/hildon/theme"));
                return slt.split(QLatin1String("/")).last();
            }
        }
    }

    return QGtkStylePrivate::getThemeName();
}

/*!
  \reimp
  NOTE: Reverse color in fremantle themes are set only for
  GtkTextView widgets. GtkEntry returns a dark base color.
*/
GtkWidget* QMaemo5StylePrivate::getTextColorWidget() const
{
    return  gtkWidget(QLS("GtkTextView"));
}


void QMaemo5StylePrivate::initGtkMenu()
{
    QGtkStylePrivate::initGtkMenu();

    //Application context menu.
    GtkWidget *gtkContainerMenu = gtk_menu_new();
    gtk_widget_set_name(gtkContainerMenu, "menu_force_with_corners");
    gtk_widget_realize(gtkContainerMenu);
    addWidgetToMap(gtkContainerMenu);
}

void QMaemo5StylePrivate::initGtkWidgets()
{
    QGtkStylePrivate::initGtkWidgets();

    if (hildon_init) {
        hildon_init();

        //ComboBoxes buttons
        addWidget(hildon_number_editor_new(0,1));
        addWidget(hildon_app_menu_new());
        addWidget(hildon_entry_new(HILDON_SIZE_FINGER_HEIGHT));
        addWidget(gtk_text_view_new());

        // normal buttons should be the "finger" size kind
        GtkWidget *gtkButton = gtkWidget(QLS("GtkButton"));
        GtkWidget *maemo5Button = hildon_button_new(HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT, HILDON_BUTTON_ARRANGEMENT_VERTICAL);
        Q_ASSERT(gtkButton);
        Q_ASSERT(maemo5Button);
        setupGtkWidget(maemo5Button);
        gtk_widget_realize(maemo5Button);

        gtkWidgetMap()->remove(QLS("GtkButton"));
        gtkWidgetMap()->insert(QLS("GtkButton"), maemo5Button);
    }
}

extern QStringList qt_make_filter_list(const QString &filter);

void QMaemo5StylePrivate::setupGtkFileChooser(GtkWidget* gtkFileChooser, QWidget *parent,
                                const QString &dir, const QString &filter, QString *,
                                QFileDialog::Options options, bool isSaveDialog,
                                QMap<GtkFileFilter *, QString> *)
{
    g_object_set(gtkFileChooser, "do-overwrite-confirmation", gboolean(!(options & QFileDialog::DontConfirmOverwrite)), NULL);
    g_object_set(gtkFileChooser, "local_only", gboolean(true), NULL);
    if (!filter.isEmpty()) {
        QStringList filters = qt_make_filter_list(filter);

        if (isSaveDialog) {
            const int Nfilters = filters.count();
            char *ext_names[Nfilters];
            char *extensions[Nfilters];

            for (int i = 0; i < Nfilters; i++) {
                const QString &rawfilter = filters[i];
                QString name = rawfilter.left(rawfilter.indexOf(QLatin1Char('(')));
                QString extension = extract_filter(rawfilter)[0].remove(QLatin1String("*."));
                //Doesn't make sense adding '*' extension in a save dialog;
                if (extension.compare(QLatin1String("*")) == 0)
                    qWarning("'*' is not a valid extension for a save dialog");
                ext_names[i]= (char*) malloc(name.count()+1);
                memcpy(ext_names[i], qPrintable(name), name.count());
                extensions[i]= (char*) malloc(extension.count()+1);
                memcpy(extensions[i], qPrintable(extension), extension.count());
            }
            ext_names[Nfilters] = NULL;
            extensions[Nfilters] = NULL;

            GtkWidget* gtkExtensionCombo;
            gtkExtensionCombo = hildon_file_chooser_dialog_add_extensions_combo((HildonFileChooserDialog*)(gtkFileChooser),
                    extensions,ext_names);
            //gtk_signal_connect (GTK_OBJECT(gtkExtensionCombo), "changed",
            //                    GTK_SIGNAL_FUNC (qt_update_file_filter), NULL);
        } else {
            //Set "*.ext" filters in the open file dialogs. There we don't have a combobox to select a filter. :( 
            //TODO the combobox in the HildonFileChooserDialog doesn't change the filter automatically. We have to se it
            //in Qt..
            GtkFileFilter *gtkFilter = gtk_file_filter_new ();
            foreach (const QString &rawfilter, filters) {
                QStringList extensions = extract_filter(rawfilter);
                foreach (const QString &fileExtension, extensions) {
                    gtk_file_filter_add_pattern (gtkFilter, qPrintable(fileExtension));
                }
            }
            g_object_set(gtkFileChooser, "filter", gtkFilter, NULL);
        }
    }

    // Using the currently active window is not entirely correct, however
    // it gives more sensible behavior for applications that do not provide a
    // parent
    QWidget *modalFor = parent ? parent->window() : qApp->activeWindow();
    if (modalFor) {
        gtk_widget_realize(gtkFileChooser); // Creates X window
        XSetTransientForHint(gdk_x11_drawable_get_xdisplay(gtkFileChooser->window),
                             gdk_x11_drawable_get_xid(gtkFileChooser->window),
                             modalFor->winId());
        gdk_x11_window_set_user_time (gtkFileChooser->window, QX11Info::appUserTime());
    }

    QFileInfo fileinfo(dir);
    if (dir.isEmpty())
        fileinfo.setFile(QDir::currentPath());
    fileinfo.makeAbsolute();
    if (fileinfo.isDir()) {
        gtk_file_chooser_set_current_folder((GtkFileChooser*)gtkFileChooser, qPrintable(dir));
    } else if (isSaveDialog) {
        gtk_file_chooser_set_current_folder((GtkFileChooser*)gtkFileChooser, qPrintable(fileinfo.absolutePath()));
        gtk_file_chooser_set_current_name((GtkFileChooser*)gtkFileChooser, qPrintable(fileinfo.fileName()));
    } else {
        gtk_file_chooser_set_filename((GtkFileChooser*)gtkFileChooser, qPrintable(dir));
    }
}

QString QMaemo5StylePrivate::openFilename(QWidget *parent, const QString &, const QString &dir, const QString &filter,
        QString *selectedFilter, QFileDialog::Options options)
{
    QMap<GtkFileFilter *, QString> filterMap;
    GtkWidget *gtkFileChooser = hildon_file_chooser_dialog_new (NULL, GTK_FILE_CHOOSER_ACTION_OPEN);

    setupGtkFileChooser(gtkFileChooser, parent, dir, filter, selectedFilter, options, false, &filterMap);

    QWidget modal_widget;
    modal_widget.setAttribute(Qt::WA_NoChildEventsForParent, true);
    modal_widget.setParent(parent, Qt::Window);
    QApplicationPrivate::enterModal(&modal_widget);

    QString filename;
    if (gtk_dialog_run ((GtkDialog*)gtkFileChooser) == GTK_RESPONSE_OK) {
        char *gtk_filename = gtk_file_chooser_get_filename ((GtkFileChooser*)gtkFileChooser);
        filename = QString::fromUtf8(gtk_filename);
        g_free (gtk_filename);
        if (selectedFilter) {
            GtkFileFilter *gtkFilter = gtk_file_chooser_get_filter ((GtkFileChooser*)gtkFileChooser);
            *selectedFilter = filterMap.value(gtkFilter);
        }
    }

    QApplicationPrivate::leaveModal(&modal_widget);
    gtk_widget_destroy (gtkFileChooser);
    return filename;
}


QString QMaemo5StylePrivate::openDirectory(QWidget *parent, const QString &, const QString &dir, QFileDialog::Options options)
{
    QMap<GtkFileFilter *, QString> filterMap;
    GtkWidget *gtkFileChooser = hildon_file_chooser_dialog_new (NULL, GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);

    setupGtkFileChooser(gtkFileChooser, parent, dir, QString(), 0, options);
    QWidget modal_widget;
    modal_widget.setAttribute(Qt::WA_NoChildEventsForParent, true);
    modal_widget.setParent(parent, Qt::Window);
    QApplicationPrivate::enterModal(&modal_widget);

    QString filename;
    if (gtk_dialog_run ((GtkDialog*)gtkFileChooser) == GTK_RESPONSE_OK) {
        char *gtk_filename = gtk_file_chooser_get_filename ((GtkFileChooser*)gtkFileChooser);
        filename = QString::fromUtf8(gtk_filename);
        g_free (gtk_filename);
    }

    QApplicationPrivate::leaveModal(&modal_widget);
    gtk_widget_destroy (gtkFileChooser);
    return filename;
}

QStringList QMaemo5StylePrivate::openFilenames(QWidget *parent, const QString &, const QString &dir, const QString &filter,
                                 QString *selectedFilter, QFileDialog::Options options)
{
    QStringList filenames;
    QMap<GtkFileFilter *, QString> filterMap;
    GtkWidget *gtkFileChooser = hildon_file_chooser_dialog_new (NULL, GTK_FILE_CHOOSER_ACTION_OPEN);

    setupGtkFileChooser(gtkFileChooser, parent, dir, filter, selectedFilter, options, false, &filterMap);
    g_object_set(gtkFileChooser, "select-multiple", gboolean(true), NULL);

    QWidget modal_widget;
    modal_widget.setAttribute(Qt::WA_NoChildEventsForParent, true);
    modal_widget.setParent(parent, Qt::Window);
    QApplicationPrivate::enterModal(&modal_widget);

    if (gtk_dialog_run ((GtkDialog*)gtkFileChooser) == GTK_RESPONSE_OK) {
        GSList *gtk_file_names = gtk_file_chooser_get_filenames((GtkFileChooser*)gtkFileChooser);
        for (GSList *iterator  = gtk_file_names ; iterator; iterator = iterator->next)
            filenames << QString::fromUtf8((const char*)iterator->data);
        g_slist_free(gtk_file_names);
        if (selectedFilter) {
            GtkFileFilter *gtkFilter = gtk_file_chooser_get_filter ((GtkFileChooser*)gtkFileChooser);
            *selectedFilter = filterMap.value(gtkFilter);
        }
    }

    QApplicationPrivate::leaveModal(&modal_widget);
    gtk_widget_destroy (gtkFileChooser);
    return filenames;
}

QString QMaemo5StylePrivate::saveFilename(QWidget *parent, const QString &, const QString &dir, const QString &filter,
                           QString *selectedFilter, QFileDialog::Options options)
{
    QMap<GtkFileFilter *, QString> filterMap;
    GtkWidget *gtkFileChooser = hildon_file_chooser_dialog_new (NULL, GTK_FILE_CHOOSER_ACTION_SAVE);
    setupGtkFileChooser(gtkFileChooser, parent, dir, filter, selectedFilter, options, true, &filterMap);

    QWidget modal_widget;
    modal_widget.setAttribute(Qt::WA_NoChildEventsForParent, true);
    modal_widget.setParent(parent, Qt::Window);
    QApplicationPrivate::enterModal(&modal_widget);

    QString filename;
    if (gtk_dialog_run ((GtkDialog*)gtkFileChooser) == GTK_RESPONSE_OK) {
        char *gtk_filename = gtk_file_chooser_get_filename ((GtkFileChooser*)gtkFileChooser);
        filename = QString::fromUtf8(gtk_filename);
        g_free (gtk_filename);
        if (selectedFilter) {
            GtkFileFilter *gtkFilter = gtk_file_chooser_get_filter ((GtkFileChooser*)gtkFileChooser);
            *selectedFilter = filterMap.value(gtkFilter);
        }
    }

    QApplicationPrivate::leaveModal(&modal_widget);
    gtk_widget_destroy (gtkFileChooser);
    return filename;
}


static QColor mergedColors(const QColor &colorA, const QColor &colorB, int factor = 50)
{
    const int maxFactor = 100;
    QColor tmp = colorA;
    tmp.setRed((tmp.red() * factor) / maxFactor + (colorB.red() * (maxFactor - factor)) / maxFactor);
    tmp.setGreen((tmp.green() * factor) / maxFactor + (colorB.green() * (maxFactor - factor)) / maxFactor);
    tmp.setBlue((tmp.blue() * factor) / maxFactor + (colorB.blue() * (maxFactor - factor)) / maxFactor);
    return tmp;
}


/*!
    \class QMaemo5Style
    \brief The QMaemo5Style class provides a widget style rendered by GTK+

    The QMaemo5Style style provides a look and feel that integrates well
    into Hildon-based desktop environments.

    It's based on QGTKStyle.

    Note: The style requires GTK+ version 2.10 or later and the special patched gtk hildon include files.
          The Qt3-based "Qt" GTK+ theme engine will not work with QGtkStyle.
*/

/*!
    Constructs a QGtkStyle object.
*/
QMaemo5Style::QMaemo5Style()
    : QGtkStyle(*new QMaemo5StylePrivate)
{
}

/*!
    Destroys the QMaemo5Style object.
*/
QMaemo5Style::~QMaemo5Style()
{
}

/*!
    \reimp
    Set the maemos style file dialog hooks.
*/
void QMaemo5Style::polish(QApplication *app)
{
    Q_D(QMaemo5Style);

    QGtkStyle::polish(app);
    if (app->desktopSettingsAware() && d->isThemeAvailable()) {
        // I want to use the maemo style dialogs
        qt_filedialog_open_filename_hook = &QMaemo5StylePrivate::openFilename;
        qt_filedialog_save_filename_hook = &QMaemo5StylePrivate::saveFilename;
        qt_filedialog_open_filenames_hook = &QMaemo5StylePrivate::openFilenames;
        qt_filedialog_existing_directory_hook = &QMaemo5StylePrivate::openDirectory;
    }
}


int QMaemo5Style::pixelMetric(PixelMetric metric,
                           const QStyleOption *option,
                           const QWidget *widget) const
{
    Q_D(const QMaemo5Style);

   if (!d->isThemeAvailable())
        return QGtkStyle::pixelMetric(metric, option, widget);

   switch (metric) {
   case PM_DialogButtonsButtonHeight:
       return 70;
   case PM_DialogButtonsButtonWidth:
       return 180;
    //coordinate of the Application Context Menu upper left corner.
    case PM_ToolBarIconSize:
        return 48;
    case PM_ButtonShiftHorizontal: {
        GtkWidget *gtkButton = d->gtkWidget(QLatin1String("GtkButton"));
        guint horizontal_shift;
        d->gtk_widget_style_get(gtkButton, "child-displacement-x", &horizontal_shift, NULL);
        return horizontal_shift;
    }
    case PM_ButtonShiftVertical: {
        GtkWidget *gtkButton = d->gtkWidget(QLatin1String("GtkButton"));
        guint vertical_shift;
        d->gtk_widget_style_get(gtkButton, "child-displacement-y", &vertical_shift, NULL);
        return vertical_shift;
    }
    /*
    case PM_MenuOffsetHorizontal:
    {
        GtkWidget *gtkMenu = d->gtkWidget(QLatin1String("menu_force_with_corners"));
        gint horizontal_offset;
        d->gtk_widget_style_get(gtkMenu, "horizontal-offset", &horizontal_offset, NULL);
        return horizontal_offset;
    }
    case PM_MenuOffsetVertical:
    {
        GtkWidget *gtkMenu = d->gtkWidget(QLatin1String("menu_force_with_corners"));
        gint vertical_offset;
        d->gtk_widget_style_get(gtkMenu, "vertical-offset", &vertical_offset, NULL);
        return vertical_offset;
    }
    */
    default:
        return  QGtkStyle::pixelMetric(metric, option, widget);
    }
}

int QMaemo5Style::styleHint(StyleHint hint, const QStyleOption *option, const QWidget *widget,
                         QStyleHintReturn *returnData = 0) const
{
    Q_D(const QMaemo5Style);

    if (!d->isThemeAvailable())
        return QGtkStyle::styleHint(hint, option, widget, returnData);

    switch (hint) {
    case SH_Menu_Scrollable:
        return int(true);
    case SH_DialogButtonBox_ButtonsHaveIcons:
    case SH_ScrollBar_ContextMenu:
        return int(false);
    case SH_UnderlineShortcut:
        return int(false);
    default:
        return QGtkStyle::styleHint(hint, option, widget, returnData);
    }
}

void QMaemo5Style::drawPrimitive(PrimitiveElement element,
                              const QStyleOption *option,
                              QPainter *painter,
                              const QWidget *widget) const
{
    Q_D(const QMaemo5Style);

    if (!d->isThemeAvailable()) {
        QGtkStyle::drawPrimitive(element, option, painter, widget);
        return;
    }

    GtkStyle* style = d->gtkStyle();
    QGtkPainter gtkPainter(painter);

    switch (element) {
    case PE_PanelButtonCommand: {
        bool isDefault = false;
        if (const QStyleOptionButton *btn = qstyleoption_cast<const QStyleOptionButton*>(option))
            isDefault = btn->features & QStyleOptionButton::DefaultButton;

        GtkStateType state = gtkPainter.gtkState(option);
        if (option->state & State_On || option->state & State_Sunken)
            state = GTK_STATE_ACTIVE;
        GtkWidget *gtkButton = d->gtkWidget(QLS("GtkButton"));
        gint focusWidth, focusPad;
        gboolean interiorFocus = false;
        d->gtk_widget_style_get (gtkButton,
                                "focus-line-width", &focusWidth,
                                "focus-padding", &focusPad,
                                "interior-focus", &interiorFocus, NULL);

        style = gtkButton->style;

        QRect buttonRect = option->rect;

        QString key;
        if (isDefault) {
            key += QLS("def");
            GTK_WIDGET_SET_FLAGS(gtkButton, GTK_HAS_DEFAULT);
            gtkPainter.paintBox(gtkButton, "buttondefault", buttonRect, state, GTK_SHADOW_IN,
                                style, isDefault ? QLS("d") : QString());
        }

        bool hasFocus = option->state & State_HasFocus;

        if (hasFocus) {
            key += QLS("def");
            GTK_WIDGET_SET_FLAGS(gtkButton, GTK_HAS_FOCUS);
        }

        if (!interiorFocus)
            buttonRect = buttonRect.adjusted(focusWidth, focusWidth, -focusWidth, -focusWidth);

        GtkShadowType shadow = (option->state & State_Sunken || option->state & State_On ) ?
                               GTK_SHADOW_IN : GTK_SHADOW_OUT;

        gtkPainter.paintBox(gtkButton, "button", buttonRect, state, shadow,
                            style, key);
        if (isDefault)
            GTK_WIDGET_UNSET_FLAGS(gtkButton, GTK_HAS_DEFAULT);
        if (hasFocus)
            GTK_WIDGET_UNSET_FLAGS(gtkButton, GTK_HAS_FOCUS);
    }
    break;

    case PE_PanelLineEdit:
        if (/*const QStyleOptionFrame *panel =*/ qstyleoption_cast<const QStyleOptionFrame *>(option)) {
            GtkWidget *gtkEntry = d->gtkWidget(QLS("HildonEntry-finger"));

            //uint resolve_mask = option->palette.resolve();

            //HildonEntry-finger could be ythickness = 0; We MUST adjust the height of the rect.
            QRect textRect = option->rect.adjusted(gtkEntry->style->xthickness,
                                                   gtkEntry->style->ythickness ? gtkEntry->style->ythickness : gtkEntry->style->xthickness,
                                                   -gtkEntry->style->xthickness,
                                                   -(gtkEntry->style->ythickness ? gtkEntry->style->ythickness : gtkEntry->style->xthickness) );


            const QLineEdit *le = qobject_cast<const QLineEdit*>(widget);

            if (le && le->hasFrame()) {
                gtkPainter.paintFlatBox(gtkEntry, "entry_bg", option->rect,
                                        option->state & State_Enabled ? GTK_STATE_NORMAL : GTK_STATE_INSENSITIVE, GTK_SHADOW_NONE, gtkEntry->style);
                drawPrimitive(PE_FrameLineEdit, option, painter, widget);
            } else {
                //QLineEdit used as Delegate widgets (Eg: in QTableView) should not have lineWidth in Hildon.
                //In this case we have to cover the widget on the back.
                QCleanlooksStyle::drawPrimitive(element, option, painter, widget);
            }

        }
        break;
    case PE_FrameLineEdit: {
        const QStyleOptionFrame *panel = qstyleoption_cast<const QStyleOptionFrame *>(option);
        if (!panel)
            break;

            //Draws a big frame
            GtkWidget *gtkEntry = d->gtkWidget(QLS("HildonEntry-finger"));

            gboolean interior_focus;
            gint focus_line_width;
            QRect rect = option->rect;
            d->gtk_widget_style_get(gtkEntry,
                                   "interior-focus", &interior_focus,
                                   "focus-line-width", &focus_line_width, NULL);

            //See https://bugzilla.mozilla.org/show_bug.cgi?id=405421 for info about this hack
            g_object_set_data(G_OBJECT(gtkEntry), "transparent-bg-hint", GINT_TO_POINTER(TRUE));

            if (!interior_focus && option->state & State_HasFocus)
                rect.adjust(focus_line_width, focus_line_width, -focus_line_width, -focus_line_width);


            gtkPainter.paintShadow(gtkEntry, "entry", rect, option->state & State_Enabled ?
                                   (option->state & State_HasFocus ? GTK_STATE_ACTIVE : GTK_STATE_NORMAL) : GTK_STATE_INSENSITIVE,
                                   GTK_SHADOW_IN, gtkEntry->style,
                                   option->state & State_HasFocus ? QLS("focus") : QString());
    }
    break;
    default:
        QGtkStyle::drawPrimitive(element, option, painter, widget);
    }
}

void QMaemo5Style::drawComplexControl(ComplexControl control, const QStyleOptionComplex *option,
                                   QPainter *painter, const QWidget *widget) const
{
    Q_D(const QMaemo5Style);

    if (!d->isThemeAvailable()) {
        QGtkStyle::drawComplexControl(control, option, painter, widget);
        return;
    }

    GtkStyle* style = d->gtkStyle();
    QGtkPainter gtkPainter(painter);
    QColor button = option->palette.button().color();
    QColor dark;
    QColor grooveColor;
    QColor darkOutline;
    dark.setHsv(button.hue(),
                qMin(255, (int)(button.saturation()*1.9)),
                qMin(255, (int)(button.value()*0.7)));
    grooveColor.setHsv(button.hue(),
                       qMin(255, (int)(button.saturation()*2.6)),
                       qMin(255, (int)(button.value()*0.9)));
    darkOutline.setHsv(button.hue(),
                       qMin(255, (int)(button.saturation()*3.0)),
                       qMin(255, (int)(button.value()*0.6)));

    QColor alphaCornerColor;

    if (widget)
        alphaCornerColor = mergedColors(option->palette.color(widget->backgroundRole()), darkOutline);
    else
        alphaCornerColor = mergedColors(option->palette.background().color(), darkOutline);

    QPalette palette = option->palette;

    switch (control) {
    #ifndef QT_NO_COMBOBOX

    case CC_ComboBox:
        // See: http://live.gnome.org/GnomeArt/Tutorials/GtkThemes/GtkComboBox
        // and http://live.gnome.org/GnomeArt/Tutorials/GtkThemes/GtkComboBoxEntry
        if (const QStyleOptionComboBox *comboBox = qstyleoption_cast<const QStyleOptionComboBox *>(option)) {
            bool sunken = comboBox->state & State_On; // play dead, if combobox has no items
            BEGIN_STYLE_PIXMAPCACHE(QString::fromLatin1("cb-%0-%1").arg(sunken).arg(comboBox->editable));
            QGtkPainter gtkCachedPainter(p);
            gtkCachedPainter.setUsePixmapCache(false); // cached externally

            bool isEnabled = (comboBox->state & State_Enabled);
            bool focus = isEnabled && (comboBox->state & State_HasFocus);
            QColor buttonShadow = option->palette.dark().color();
            GtkStateType state = gtkPainter.gtkState(option);
            int appears_as_list = !styleHint(QStyle::SH_ComboBox_Popup, comboBox, widget);
            QPixmap cache;
            QString pixmapName;
            QStyleOptionComboBox comboBoxCopy = *comboBox;
            comboBoxCopy.rect = option->rect;

            bool reverse = (option->direction == Qt::RightToLeft);
            QRect rect = option->rect;
            QRect arrowButtonRect = subControlRect(CC_ComboBox, &comboBoxCopy,
                                                   SC_ComboBoxArrow, widget);
            QRect editRect = subControlRect(CC_ComboBox, &comboBoxCopy,
                                            SC_ComboBoxEditField, widget);

            GtkShadowType shadow = (option->state & State_Sunken || option->state & State_On ) ?
                                   GTK_SHADOW_IN : GTK_SHADOW_OUT;
            QString comboBoxPath = QLS(comboBox->editable ? "GtkComboBoxEntry" : "GtkComboBox");

            // We use the gtk widget to position arrows and separators for us
            GtkWidget *gtkCombo = d->gtkWidget(comboBoxPath);
            GtkAllocation geometry = {0, 0, option->rect.width(), option->rect.height()};
            d->gtk_widget_set_direction(gtkCombo, reverse ? GTK_TEXT_DIR_RTL : GTK_TEXT_DIR_LTR);
            d->gtk_widget_size_allocate(gtkCombo, &geometry);

            QString buttonPath = comboBoxPath + QLS(".GtkToggleButton");
            GtkWidget *gtkToggleButton = d->gtkWidget(buttonPath);
            d->gtk_widget_set_direction(gtkToggleButton, reverse ? GTK_TEXT_DIR_RTL : GTK_TEXT_DIR_LTR);
            if (gtkToggleButton && (appears_as_list || comboBox->editable)) {
                if (focus)
                    GTK_WIDGET_SET_FLAGS(gtkToggleButton, GTK_HAS_FOCUS);
                // Draw the combo box as a line edit with a button next to it
                if (comboBox->editable || appears_as_list) {
                    GtkStateType frameState = (state == GTK_STATE_PRELIGHT) ? GTK_STATE_NORMAL : state;
                    QString entryPath = QLS(comboBox->editable ? "GtkComboBoxEntry.GtkEntry" : "GtkComboBox.GtkFrame");
                    GtkWidget *gtkEntry = d->gtkWidget(entryPath);
                    d->gtk_widget_set_direction(gtkEntry, reverse ? GTK_TEXT_DIR_RTL : GTK_TEXT_DIR_LTR);
                    QRect frameRect = option->rect;

                    if (reverse)
                        frameRect.setLeft(arrowButtonRect.right());
                    else
                        frameRect.setRight(arrowButtonRect.left());

                    // uint resolve_mask = option->palette.resolve();
                    int xt = gtkEntry->style->xthickness;
                    int yt = gtkEntry->style->ythickness;
                    QRect contentRect = frameRect.adjusted(xt, yt, -xt, -yt);

                    if (comboBox->editable) {
                        gtkCachedPainter.paintFlatBox(gtkEntry, "entry_bg", contentRect,
                                                      option->state & State_Enabled ? GTK_STATE_ACTIVE : GTK_STATE_INSENSITIVE,
                                                      GTK_SHADOW_NONE, gtkEntry->style, entryPath + QString::number(focus));
                    } else {
                        //Evil hack used by Gtkrc
                        GdkColor gdkBg;
                        QColor bg;
                        gdkBg = gtkCombo->style->base[option->state & State_Enabled ? GTK_STATE_NORMAL : GTK_STATE_INSENSITIVE];
                        bg = QColor(gdkBg.red>>8, gdkBg.green>>8, gdkBg.blue>>8);
                        p->fillRect(contentRect, bg);
                    }

                    gtkCachedPainter.paintShadow(gtkEntry, comboBox->editable ? "entry" : "frame", frameRect, frameState,
                                           GTK_SHADOW_IN, gtkEntry->style, entryPath +
                                           QString::number(focus) + QString::number(comboBox->editable) +
                                           QString::number(option->direction));
                }

                GtkStateType buttonState = GTK_STATE_NORMAL;

                if (!(option->state & State_Enabled))
                    buttonState = GTK_STATE_INSENSITIVE;
                else if (option->state & State_Sunken || option->state & State_On)
                    buttonState = GTK_STATE_ACTIVE;
                else if (option->state & State_MouseOver && comboBox->activeSubControls & SC_ComboBoxArrow)
                    buttonState = GTK_STATE_PRELIGHT;

                QRect buttonrect = QRect(gtkToggleButton->allocation.x, gtkToggleButton->allocation.y,
                                         gtkToggleButton->allocation.width, gtkToggleButton->allocation.height);

                Q_ASSERT(gtkToggleButton);
                gtkCachedPainter.paintBox( gtkToggleButton, "button", arrowButtonRect, buttonState,
                                     shadow, gtkToggleButton->style, buttonPath +
                                     QString::number(focus) + QString::number(option->direction));
                if (focus)
                    GTK_WIDGET_UNSET_FLAGS(gtkToggleButton, GTK_HAS_FOCUS);
            } else {
                // Draw combo box as a button
                QRect buttonRect = option->rect;

                if (focus) // Clearlooks actually check the widget for the default state
                    GTK_WIDGET_SET_FLAGS(gtkToggleButton, GTK_HAS_FOCUS);
                gtkCachedPainter.paintBox(gtkToggleButton, "button",
                                    buttonRect, state,
                                    shadow, gtkToggleButton->style,
                                    buttonPath + QString::number(focus));
                if (focus)
                    GTK_WIDGET_UNSET_FLAGS(gtkToggleButton, GTK_HAS_FOCUS);

                // Draw the separator between label and arrows
                QString vSeparatorPath = buttonPath + QLS(".GtkHBox.GtkVSeparator");

                if (GtkWidget *gtkVSeparator = d->gtkWidget(vSeparatorPath)) {
                    QRect vLineRect(gtkVSeparator->allocation.x,
                                    gtkVSeparator->allocation.y,
                                    gtkVSeparator->allocation.width,
                                    gtkVSeparator->allocation.height);

                    gtkCachedPainter.paintVline( gtkVSeparator, "vseparator",
                                           vLineRect, state, gtkVSeparator->style,
                                           0, vLineRect.height(), 0,  vSeparatorPath);


                    gint interiorFocus = true;
                    d->gtk_widget_style_get(gtkToggleButton, "interior-focus", &interiorFocus, NULL);
                    int xt = interiorFocus ? gtkToggleButton->style->xthickness : 0;
                    int yt = interiorFocus ? gtkToggleButton->style->ythickness : 0;
                    if ((focus && (option->state & State_KeyboardFocusChange)))
                        gtkCachedPainter.paintFocus(gtkToggleButton, "button",
                                              option->rect.adjusted(xt, yt, -xt, -yt),
                                              option->state & State_Sunken ? GTK_STATE_ACTIVE : GTK_STATE_NORMAL,
                                              gtkToggleButton->style);
                }
            }

            if (comboBox->subControls & SC_ComboBoxArrow) {
                if (!isEnabled)
                    state = GTK_STATE_INSENSITIVE;
                else if (sunken)
                    state = GTK_STATE_ACTIVE;
                else if (option->state & State_MouseOver)
                    state = GTK_STATE_PRELIGHT;
                else
                    state = GTK_STATE_NORMAL;

                QString arrowPath = comboBoxPath + QLS(appears_as_list ? ".GtkToggleButton.GtkArrow"
                                                    : ".GtkToggleButton.GtkHBox.GtkArrow");

                GtkWidget *gtkArrow = d->gtkWidget(arrowPath);
                gfloat scale = 0.7;
                gint minSize = 15;
                QRect arrowWidgetRect;

                if (gtkArrow && !d->gtk_check_version(2, 12, 0)) {
                    d->gtk_widget_style_get(gtkArrow, "arrow-scaling", &scale, NULL);
                    d->gtk_widget_style_get(gtkCombo, "arrow-size", &minSize, NULL);
                }
                if (gtkArrow) {
                    arrowWidgetRect = QRect(gtkArrow->allocation.x, gtkArrow->allocation.y,
                                            gtkArrow->allocation.width, gtkArrow->allocation.height);
                    style = gtkArrow->style;
                }

                // Note that for some reason the arrow-size is not properly respected with Hildon
                // Hence we enforce the minimum "arrow-size" ourselves
                int arrowSize = qMax(qMin(rect.height() - gtkCombo->style->ythickness * 2, minSize),
                                     qMin(arrowWidgetRect.width(), arrowWidgetRect.height()));
                QRect arrowRect(0, 0, static_cast<int>(arrowSize * scale), static_cast<int>(arrowSize * scale));

                arrowRect.moveCenter(arrowWidgetRect.center());

                if (sunken) {
                    int xoff, yoff;
                    GtkWidget *gtkButton = d->gtkWidget(comboBoxPath + QLS(".GtkToggleButton"));
                    d->gtk_widget_style_get(gtkButton, "child-displacement-x", &xoff, NULL);
                    d->gtk_widget_style_get(gtkButton, "child-displacement-y", &yoff, NULL);
                    arrowRect = arrowRect.adjusted(xoff, yoff, xoff, yoff);
                }

                // Some styles such as Nimbus paint outside the arrowRect
                // hence we have provide the whole widget as the cliprect
                if (gtkArrow) {
                    gtkCachedPainter.setClipRect(option->rect);
                    gtkCachedPainter.paintArrow( gtkArrow, "arrow", arrowRect,
                                           GTK_ARROW_DOWN, state, GTK_SHADOW_NONE, TRUE,
                                           style, arrowPath + QString::number(option->direction));
                }
            }
            END_STYLE_PIXMAPCACHE;
        }
        break;
#endif // QT_NO_COMBOBOX

#ifndef QT_NO_SPINBOX
    // Note: according to Maemo UI guidelines the gtkspinbutton is deprecated
    case CC_SpinBox:
        if (const QStyleOptionSpinBox *spinBox = qstyleoption_cast<const QStyleOptionSpinBox *>(option)) {
            // GtkWidget *gtkSpinButton = d->gtkWidget(QLS("GtkSpinButton"));
            bool isEnabled = (spinBox->state & State_Enabled);
            // bool hover = isEnabled && (spinBox->state & State_MouseOver);
            bool sunken = (spinBox->state & State_Sunken);
            bool upIsActive = (spinBox->activeSubControls == SC_SpinBoxUp);
            bool downIsActive = (spinBox->activeSubControls == SC_SpinBoxDown);
            // bool reverse = (spinBox->direction == Qt::RightToLeft);

            GtkWidget *gtkEntry = d->gtkWidget(QLatin1String("HildonNumberEditor.GtkEntry"));
            GtkWidget *gtkMinusButton = d->gtkWidget(QLatin1String("HildonNumberEditor.ne-minus-button"));
            GtkWidget *gtkPlusButton = d->gtkWidget(QLatin1String("HildonNumberEditor.ne-plus-button"));
            int xt = gtkEntry->style->xthickness;
            int yt = gtkEntry->style->ythickness;
            QRect plusRect = subControlRect(CC_SpinBox, option, SC_SpinBoxUp, widget);
            QRect minusRect = subControlRect(CC_SpinBox, option, SC_SpinBoxDown, widget);
            QRect entryRect = subControlRect(CC_SpinBox, option, SC_SpinBoxEditField, widget).adjusted(-xt, -yt, xt, yt);

            /* TODO: Do not replicate code from PE_FrameLineEdit */
            gboolean interior_focus;
            gint focus_line_width;
            QRect rect = entryRect;
            d->gtk_widget_style_get(gtkEntry,
                                "interior-focus", &interior_focus,
                                "focus-line-width", &focus_line_width,
                                NULL);

            if (option->state & State_HasFocus)
                GTK_WIDGET_SET_FLAGS(gtkEntry, GTK_HAS_FOCUS);
            else
                GTK_WIDGET_UNSET_FLAGS(gtkEntry, GTK_HAS_FOCUS);

#if defined Q_WS_HILDON && !defined Q_OS_FREMANTLE
            gtkPainter.paintFlatBox(gtkEntry, "entry_bg", entryRect, GTK_STATE_NORMAL, GTK_SHADOW_NONE, gtkEntry->style);

            if (!interior_focus && option->state & State_HasFocus)
                rect.adjust(focus_line_width, focus_line_width, -focus_line_width, -focus_line_width);

            gtkPainter.paintShadow(gtkEntry, "entry", option->rect, GTK_STATE_NORMAL, GTK_SHADOW_IN, gtkEntry->style);

            if (!interior_focus && option->state & State_HasFocus)
                gtkPainter.paintShadow(gtkEntry, "entry", option->rect, GTK_STATE_ACTIVE, GTK_SHADOW_IN, gtkEntry->style);
#endif
            /* Plus button */
            GtkStateType state = (upIsActive && sunken) ? GTK_STATE_ACTIVE : GTK_STATE_NORMAL;
            if (!isEnabled)
                state = GTK_STATE_INSENSITIVE;

            gtkPainter.paintBox(gtkPlusButton, "button",  plusRect, state, GTK_SHADOW_OUT, gtkPlusButton->style);

            /* Minus button */
            state = (downIsActive && sunken) ? GTK_STATE_ACTIVE : GTK_STATE_NORMAL;
            if (!isEnabled)
                state = GTK_STATE_INSENSITIVE;
            gtkPainter.paintBox(gtkMinusButton, "button",  minusRect, state, GTK_SHADOW_OUT, gtkMinusButton->style);
        }
        break;
#endif // QT_NO_SPINBOX

    default:
        QGtkStyle::drawComplexControl(control, option, painter, widget);
    }
}

QRect QMaemo5Style::subControlRect(ComplexControl control, const QStyleOptionComplex *option,
                                SubControl subControl, const QWidget *widget) const
{
    Q_D(const QMaemo5Style);

    QRect rect = QWindowsStyle::subControlRect(control, option, subControl, widget);
    if (!d->isThemeAvailable())
        return QGtkStyle::subControlRect(control, option, subControl, widget);

    switch (control) {
#ifndef QT_NO_SPINBOX
    // Note: according to Maemo UI guidelines the gtkspinbutton is deprecated
    case CC_SpinBox:
        if (const QStyleOptionSpinBox *spinbox = qstyleoption_cast<const QStyleOptionSpinBox *>(option)) {
            GtkWidget *gtkEntry = d->gtkWidget(QLatin1String("HildonNumberEditor.GtkEntry"));
            int buttonSize = 30;
            int padding = 2;
            int h = qMax(spinbox->rect.height(), buttonSize);
            int w = qMax(spinbox->rect.width(), (buttonSize + padding) * 2);
            int x = spinbox->rect.x();
            int y = spinbox->rect.y();
            int buttonY = y + (h - buttonSize) / 2;
            int xt = gtkEntry->style->xthickness;
            int yt = gtkEntry->style->ythickness;

            switch (subControl) {
            case SC_SpinBoxUp:
                rect = QRect(x + w - buttonSize, buttonY, buttonSize, buttonSize);
                break;
            case SC_SpinBoxDown:
                rect = QRect(x, buttonY, buttonSize, buttonSize);
                break;
            case SC_SpinBoxEditField:
                rect = QRect(x + buttonSize + padding + xt, y + yt, w - 2 * (buttonSize + padding + xt), h - 2 * yt);
                break;
            case SC_SpinBoxFrame:
                rect = spinbox->rect;
            default:
                break;
            }
            rect = visualRect(spinbox->direction, spinbox->rect, rect);
        }
        break;
#endif // Qt_NO_SPINBOX
    default:
        return  QGtkStyle::subControlRect(control, option, subControl, widget);
    }
    return rect;
}

QSize QMaemo5Style::sizeFromContents(ContentsType type, const QStyleOption *option,
                                  const QSize &size, const QWidget *widget) const
{
    Q_D(const QMaemo5Style);

    QSize newSize = QGtkStyle::sizeFromContents(type, option, size, widget);
    if (!d->isThemeAvailable())
        return newSize;

    GtkWidget *gtkWidget = 0;

    switch (type) {
    case CT_LineEdit: {
        const QLineEdit *le = qobject_cast<const QLineEdit*>(widget);
        if (le && !le->hasFrame())
            break;
        gtkWidget = d->gtkWidget(QLS("HildonEntry-finger"));
        newSize = size + QSize(2*gtkWidget->style->xthickness,
                               2*qMax(gtkWidget->style->ythickness, gtkWidget->style->xthickness));
        break;
    }

    case CT_PushButton: {
        gtkWidget = d->gtkWidget(QLS("GtkButton"));
        break;
    }

#ifndef QT_NO_SPINBOX
    // Note: according to Maemo UI guidelines the gtkspinbutton is deprecated
    case CT_SpinBox: {
        gtkWidget = d->gtkWidget(QLS("HildonEntry-finger"));
        break;
    }
#endif

    default:
        break;
    }

    if (gtkWidget) {
        gint w = -1, h = -1;
        d->gtk_widget_get_size_request(gtkWidget, &w, &h);
        newSize.setHeight(qMax(newSize.height(), h));
        newSize.setWidth(qMax(newSize.width(), w));
    }
    return newSize;
}

QRect QMaemo5Style::subElementRect(SubElement element, const QStyleOption *option, const QWidget *widget) const
{
    Q_D(const QMaemo5Style);

    QRect r;
    switch (element) {
    case SE_LineEditContents: {
        const QLineEdit *le = qobject_cast<const QLineEdit*>(widget);
        if (le && !le->hasFrame())
            return  QGtkStyle::subElementRect(element, option, widget);

        if (const QStyleOptionFrame *f = qstyleoption_cast<const QStyleOptionFrame *>(option)) {
            GtkWidget *gtkEntry = d->gtkWidget(QLS("HildonEntry-finger"));
            int x = qMax(gtkEntry->style->xthickness, f->lineWidth);
            int y = qMax(gtkEntry->style->ythickness, f->lineWidth);
            r = f->rect.adjusted(x,y, -x, -y);
            r = visualRect(option->direction, option->rect, r);
        }
    }
    break;

    default:
        return  QGtkStyle::subElementRect(element, option, widget);
    }
    return r;
}


QT_END_NAMESPACE

#endif

